/*
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>
 *
 *   Copyright (C) 2008 by Javier Goday <jgoday@gmail.com>
 *   Copyright (C) 2009 by gerdfleischer <gerdfleischer@web.de>
 */

#ifndef EVENTAPPLET_H
#define EVENTAPPLET_H

// kde ui autogenerated file
#include "ui_eventappletformatconfig.h"
#include "ui_eventappletcolorconfig.h"

// Qt headers
#include <QMap>
#include <QGraphicsSceneHoverEvent>


// Plasma includes
#include <Plasma/PopupApplet>
#include <Plasma/TreeView>
#include <Plasma/Label>
#include <Plasma/BusyWidget>
#include <Plasma/ToolTipManager>

#include <akonadi/agentmanager.h>

class QDateTime;
class QTimer;
class QModelIndex;
class QAction;
class KConfigDialog;
class EventModel;
class EventFilterModel;
class EventItemDelegate;
class QGraphicsLinearLayout;
class QGraphicsProxyWidget;
class QTreeView;
class EventTreeView;

class EventApplet : public Plasma::PopupApplet
{
    Q_OBJECT

public:
    EventApplet(QObject *parent, const QVariantList &args);
    ~EventApplet();

    void init();
    QGraphicsWidget *graphicsWidget();
    virtual QList<QAction *> contextualActions();

private slots:
    void slotOpenEvent(const QModelIndex &index);
    void slotUpdateTooltip(QString);
    void openEventFromMenu();
    void slotAddEvent();
    void timerExpired();
    void setShownResources();
    void setupModel();
    void akonadiStatusChanged();

protected slots:
    void configAccepted();

protected:
    void createConfigurationInterface(KConfigDialog *parent);
    void hoverMoveEvent(QGraphicsSceneHoverEvent *event);

private:
    void colorizeModel(bool timerTriggered);
    void createToolTip();
    
private:
    QGraphicsWidget *m_graphicsWidget;
    QGraphicsLinearLayout *layout;
    EventModel *m_model;
    EventFilterModel *m_filterModel;
//    Plasma::TreeView *m_view;
    QGraphicsProxyWidget *proxyWidget;
    EventTreeView *m_view;
    Plasma::ToolTipContent tooltip;
    Plasma::BusyWidget *busy;
    EventItemDelegate *m_delegate;
    Ui::EventAppletFormatConfig m_formatConfigUi;
    Ui::EventAppletColorConfig m_colorConfigUi;
    int m_urgency, m_birthdayUrgency, m_checkInterval, m_period;
    QColor m_urgentBg, m_passedFg, m_birthdayBg, m_anniversaryBg, m_todoBg, m_finishedTodoBg;
    QList<QColor> m_colors;
    QTimer *m_timer;
    Akonadi::AgentManager *m_agentManager;
    QStringList disabledResources;
    QDateTime lastCheckTime;
    bool m_showFinishedTodos;
};

#endif
